#!/usr/bin/perl -I/usr/sausalito/perl -I/usr/sausalito/handlers/base/email/
# $Id: enable.pl

use Sauce::Service;
use Sauce::Util;
use CCE;
use Email;
use lib qw(/usr/sausalito/perl /usr/sausalito/handlers/base/network /usr/sausalito/handlers/base/email/);
use Network qw(find_eth_ifaces get_primary_interface get_device_mac check_if_slave 
                    reinitialize_network get_nmcli_uuid get_uuid_by_device 
                    get_secondary_ipv4_addresses get_secondary_ipv6_addresses calcnetwork 
                    find_config_file netmask_to_prefix get_primary_ipv4_ip 
                    get_primary_ipv4_netmask get_primary_ipv6_ip nm_debug_msg 
                    get_primary_ipv4_gateway get_primary_ipv6_gateway get_primary_dns_servers
                    remove_prefixes array_to_string string_to_array network_device_change_state
                    network_device_check_state remove_connections get_dns_servers compare_hashes 
                    in_array compare_network_configs check_dhcp blueonyx_dhcp
                    );

# Debugging switch:
$DEBUG = "0";
if ($DEBUG) {
    use Sys::Syslog qw( :DEFAULT setlogsock);
}

my $cce = new CCE( Namespace => 'Email', Domain => 'base-email' );

$cce->connectfd();

# Get 'System' Object:
my @oids = $cce->find('System');
if (not @oids) {
    $cce->bye('FAIL');
    exit 1;
}
my ($ok, $System) = $cce->get($oids[0]);
unless ($ok and $System) {
    $cce->bye('FAIL');
    exit 1;
}

# Get 'System' . 'Email' Object/Namespace:
my ($ok, $obj) = $cce->get($oids[0], 'Email');
unless ($ok and $obj) {
    $cce->bye('FAIL');
    exit 1;
}

&debug_msg("Running email/enable.pl\n");

# dovecot settings first
Sauce::Util::editfile('/etc/dovecot/dovecot.conf', *make_dovecot_conf, $obj, $System );

# Edit /etc/dovecot/conf.d/10-master.conf:
Sauce::Util::editfile('/etc/dovecot/conf.d/10-master.conf', *edit_master_conf, $obj, $System);

# Make sure we have our modified 10-ssl.conf in place:
if ((-f '/etc/dovecot/conf.d/10-ssl.conf') && (-f '/etc/dovecot/conf.d/10-ssl.conf.bx')) {
    $check = `cat /etc/dovecot/conf.d/10-ssl.conf|grep '^ssl_dh ='|wc -l`;
    chomp($check);
    if ($check eq "0") {
        system("cp /etc/dovecot/conf.d/10-ssl.conf.bx /etc/dovecot/conf.d/10-ssl.conf");
    }
}

# Stop Dovecot if all related services are turned off:
if (($obj->{'enablePop'} eq '0') && ($obj->{'enablePops'} eq '0') && ($obj->{'enableImap'} eq '0') && ($obj->{'enableImaps'} eq '0')) {
    Sauce::Service::service_toggle_init('dovecot', '0');
    Sauce::Service::service_run_init('dovecot', '0');
}
else {
    # Enable Dovecot:
    system("/sbin/chkconfig --add dovecot");
    Sauce::Service::service_set_init('dovecot', 'on', '345');
    Sauce::Service::service_toggle_init('dovecot', '1');
    Sauce::Service::service_run_init('dovecot', '1');    
}

# settings smtp, smtps and submission port
Sauce::Util::editfile(Email::SendmailMC, *make_sendmail_mc, $obj, $System );

# Rebuilding sendmail.cf:
&debug_msg("Rebuilding sendmail.cf from sendmail.mc \n");
system("m4 /usr/share/sendmail-cf/m4/cf.m4 /etc/mail/sendmail.mc > /etc/mail/sendmail.cf");

# need to start sendmail?
my $run = 0;
if ($obj->{enableSMTP} || $obj->{enableSMTPS} || $obj->{enableSubmissionPort}) {
    $run = 1;
}
if ($run) {
    Sauce::Service::service_toggle_init('sendmail', 1);
    Sauce::Service::service_toggle_init('saslauthd', $obj->{enableSMTPAuth});
} else {
    Sauce::Service::service_toggle_init('sendmail', 0);
    Sauce::Service::service_toggle_init('saslauthd', 0);
}


# pop-before-smtp relaying
my $popRelay = Sauce::Service::service_get_init('poprelayd') ? 'on' : 'off';
my $newpopRelay = $obj->{popRelay} ? 'on' : 'off';

&debug_msg("Think poprelayd is running? $popRelay - Should be? $newpopRelay\n");

Sauce::Service::service_toggle_init('poprelayd', $obj->{popRelay}); 

if($newpopRelay eq 'on') {
    &debug_msg("Linking custodiat into place\n");
    Sauce::Util::linkfile('/usr/local/sbin/poprelayd.custodiat', '/etc/cron.quarter-daily/poprelayd.custodiat');
} else {
    &debug_msg("Unlinking custodiat\n");
    Sauce::Util::unlinkfile('/etc/cron.quarter-daily/poprelayd.custodiat');
}

Sauce::Service::service_restart_xinetd();
# running swatch
#system('/usr/sbin/swatch -c /etc/swatch.conf &');

$cce->bye("SUCCESS");
exit(0);


sub edit_master_conf {

    &debug_msg("Running 'edit_master_conf')\n");

    my $fin  = shift;
    my $fout = shift;
    my $obj = shift;
    my $System = shift;

    print $fout "default_process_limit = " . $obj->{'default_process_limit'} . "\n";
    print $fout "default_client_limit = " . $obj->{'default_client_limit'} . "\n";
    print $fout "\n";
    print $fout "# Default VSZ (virtual memory size) limit for service processes. This is mainly\n";
    print $fout "# intended to catch and kill processes that leak memory before they eat up\n";
    print $fout "# everything.\n";
    print $fout "default_vsz_limit = " . $obj->{'default_vsz_limit'} . "M\n";
    print $fout "\n";
    print $fout "# Login user is internally used by login processes. This is the most untrusted\n";
    print $fout "# user in Dovecot system. It shouldn't have access to anything at all.\n";
    print $fout "default_login_user = dovenull\n";
    print $fout "\n";
    print $fout "# Internal user is used by unprivileged processes. It should be separate from\n";
    print $fout "# login user, so that login processes can't disturb other processes.\n";
    print $fout "default_internal_user = dovecot\n";
    print $fout "\n";
    print $fout "service imap-login {\n";
    print $fout "  inet_listener imap {\n";
    if ($obj->{enableImap}) {
        print $fout "    port = 143\n";
    }
    else {
        print $fout "    port = 0\n";
    }
    print $fout "  }\n";
    print $fout "  inet_listener imaps {\n";
    if ($obj->{enableImaps}) {
        print $fout "    port = 993\n";
        print $fout "    ssl = yes\n";
    }
    else {
        print $fout "    port = 0\n";
        print $fout "    ssl = no\n";
    }
    print $fout "  }\n";
    print $fout "\n";
    print $fout "  # Number of connections to handle before starting a new process. Typically\n";
    print $fout "  # the only useful values are 0 (unlimited) or 1. 1 is more secure, but 0\n";
    print $fout "  # is faster. <doc/wiki/LoginProcess.txt>\n";
    print $fout "  service_count = 1\n";
    print $fout "\n";
    print $fout "  # Number of processes to always keep waiting for more connections.\n";
    print $fout "  process_min_avail = 0\n";
    print $fout "\n";
    print $fout "  # If you set service_count=0, you probably need to grow this.\n";
    print $fout "  vsz_limit = \$default_vsz_limit\n";
    print $fout "}\n";
    print $fout "\n";
    print $fout "service pop3-login {\n";
    print $fout "  inet_listener pop3 {\n";

    if ($obj->{enablePop}) {
        print $fout "    port = 110\n";
    }
    else {
        print $fout "    port = 0\n";
    }
    print $fout "  }\n";
    print $fout "  inet_listener pop3s {\n";
    if ($obj->{enablePops}) {
        print $fout "    port = 995\n";
        print $fout "    ssl = yes\n";
    }
    else {
        print $fout "    port = 0\n";
        print $fout "    ssl = no\n";
    }
    print $fout "  }\n";
    print $fout "}\n";
    print $fout "\n";
    print $fout "service lmtp {\n";
    print $fout "  unix_listener lmtp {\n";
    print $fout "    #mode = 0666\n";
    print $fout "  }\n";
    print $fout "\n";
    print $fout "  # Create inet listener only if you can't use the above UNIX socket\n";
    print $fout "  #inet_listener lmtp {\n";
    print $fout "    # Avoid making LMTP visible for the entire internet\n";
    print $fout "    #address =\n";
    print $fout "    #port = \n";
    print $fout "  #}\n";
    print $fout "}\n";
    print $fout "\n";
    print $fout "service imap {\n";
    print $fout "  # Most of the memory goes to mmap()ing files. You may need to increase this\n";
    print $fout "  # limit if you have huge mailboxes.\n";
    print $fout "  #vsz_limit = \$default_vsz_limit\n";
    print $fout "\n";
    print $fout "  # Max. number of IMAP processes (connections)\n";
    print $fout "  process_limit = " . $obj->{'process_limit'} . "\n";
    print $fout "}\n";
    print $fout "\n";
    print $fout "service pop3 {\n";
    print $fout "  # Max. number of POP3 processes (connections)\n";
    print $fout "  process_limit = " . $obj->{'process_limit'} . "\n";
    print $fout "}\n";
    print $fout "\n";
    print $fout "service auth {\n";
    print $fout "  # auth_socket_path points to this userdb socket by default. It's typically\n";
    print $fout "  # used by dovecot-lda, doveadm, possibly imap process, etc. Users that have\n";
    print $fout "  # full permissions to this socket are able to get a list of all usernames and\n";
    print $fout "  # get the results of everyone's userdb lookups.\n";
    print $fout "  #\n";
    print $fout "  # The default 0666 mode allows anyone to connect to the socket, but the\n";
    print $fout "  # userdb lookups will succeed only if the userdb returns an \"uid\" field that\n";
    print $fout "  # matches the caller process's UID. Also if caller's uid or gid matches the\n";
    print $fout "  # socket's uid or gid the lookup succeeds. Anything else causes a failure.\n";
    print $fout "  #\n";
    print $fout "  # To give the caller full permissions to lookup all users, set the mode to\n";
    print $fout "  # something else than 0666 and Dovecot lets the kernel enforce the\n";
    print $fout "  # permissions (e.g. 0777 allows everyone full permissions).\n";
    print $fout "  unix_listener auth-userdb {\n";
    print $fout "    #mode = 0666\n";
    print $fout "    #user = \n";
    print $fout "    #group = \n";
    print $fout "  }\n";
    print $fout "\n";
    print $fout "  # Postfix smtp-auth\n";
    print $fout "  #unix_listener /var/spool/postfix/private/auth {\n";
    print $fout "  #  mode = 0666\n";
    print $fout "  #}\n";
    print $fout "\n";
    print $fout "  # Auth process is run as this user.\n";
    print $fout "  #user = \$default_internal_user\n";
    print $fout "}\n";
    print $fout "\n";
    print $fout "service auth-worker {\n";
    print $fout "  # Auth worker process is run as root by default, so that it can access\n";
    print $fout "  # /etc/shadow. If this isn't necessary, the user should be changed to\n";
    print $fout "  # \$default_internal_user.\n";
    print $fout "  #user = root\n";
    print $fout "}\n";
    print $fout "\n";
    print $fout "service dict {\n";
    print $fout "  # If dict proxy is used, mail processes should have access to its socket.\n";
    print $fout "  # For example: mode=0660, group=vmail and global mail_access_groups=vmail\n";
    print $fout "  unix_listener dict {\n";
    print $fout "    #mode = 0600\n";
    print $fout "    #user = \n";
    print $fout "    #group = \n";
    print $fout "  }\n";
    print $fout "}\n";

    return 1;
}


sub make_dovecot_conf
{
    my $in  = shift;
    my $out = shift;

    my $obj = shift;
    my $System = shift;

    my $protocols;
    # Assume safe defaults: IPv4 only.
    my $listen = 'listen = *';

    if (($System->{gateway} ne "") && ($System->{gateway_IPv6} ne "")) {
        # Dual Stack:
        $listen = 'listen = *,[::]';
    }
    elsif (($System->{gateway} eq "") && ($System->{gateway_IPv6} ne "")) {
        # IPv6 only:
        $listen = 'listen = [::]';
    }
    else {
        # IPv4 only:
        $listen = 'listen = *';
    }

    if (($obj->{enableImap}) || ($obj->{enableImaps})) {
        $protocols .= " imap";
    }

    if (($obj->{enablePop}) || ($obj->{enablePops})) {
        $protocols .= " pop3";
    }

    select $out;
    while (<$in>) {
        if (/protocols =/o) {
            print "protocols =$protocols\n";
        }
        elsif (/^listen/) {
            print $listen . "\n";
        }
        else {
            print $_;
        }
    }
    return 1;
}

sub make_sendmail_mc
{
    my $in  = shift;
    my $out = shift;

    my $obj = shift;
    my $System = shift;

    # Get primary network interface:
    my $REAL_PRIMARY_INTERFACE_NAME = get_primary_interface();

    # Get primary IPv6 IP:
    $ipv6_ip = `LC_ALL=C ip -6 addr show dev $REAL_PRIMARY_INTERFACE_NAME|grep inet6|grep -v '^fe80::/'|head -1|awk '{ print \$2 }'|cut -d / -f1`;
    chomp($ipv6_ip);

    #
    ## Cheat Sheet:
    #
    # IPv4 only:
    # DAEMON_OPTIONS(`Port=smtp, Name=MTA')
    # DAEMON_OPTIONS(`Port=submission, Name=MSA, M=Ea')
    # DAEMON_OPTIONS(`Port=smtps, Name=TLSMTA, M=s')
    # 
    # IPv6 only:
    # DAEMON_OPTIONS(`Familiy=inet6, Port=smtp, Name=MTA, Modifier=O')
    # DAEMON_OPTIONS(`Familiy=inet6, Port=submission, Name=MSA, M=Ea, Modifier=O')
    # DAEMON_OPTIONS(`Familiy=inet6, Port=smtps, Name=TLSMTA, M=s, Modifier=O')
    # 
    # IPv6-Part in Dual Stack. Requires IPv4 part as well. And yes: We can only bind to the primary IPv6 IP, not all of them:
    # DAEMON_OPTIONS(`Familiy=inet6, port=smtp, Name=MTA-v6, Modifier=O, Addr=2001:470:1f0e:7ee::30')
    # DAEMON_OPTIONS(`Familiy=inet6, Port=submission, Name=MSA-v6, M=Ea, Modifier=O, Addr=2001:470:1f0e:7ee::30')
    # DAEMON_OPTIONS(`Familiy=inet6, Port=smtps, Name=TLSMTA-v6, M=s, Modifier=O, Addr=2001:470:1f0e:7ee::30')

    # Start with empty IPv6 related lines:
    my $ipv6_part = '';

    if (($System->{gateway} ne "") && ($System->{gateway_IPv6} ne "")) {
        #
        ### Dual Stack:
        #

        # smtp port
        if ($obj->{enableSMTP}) {
            $smtpPort = "DAEMON_OPTIONS(`Port=smtp, Name=MTA')\n";
            $ipv6_part .= "DAEMON_OPTIONS(`Familiy=inet6, port=smtp, Name=MTA-v6, Modifier=O, Addr=$ipv6_ip')\n";
        }
        else {
            $smtpPort = "dnl DAEMON_OPTIONS(`Port=smtp, Name=MTA')\n";
        }

        # smtps port
        if ($obj->{enableSMTPS}) {
            $smtpsPort = "DAEMON_OPTIONS(`Port=smtps, Name=TLSMTA, M=s')\n";
            $ipv6_part .= "DAEMON_OPTIONS(`Familiy=inet6, Port=smtps, Name=TLSMTA-v6, M=s, Modifier=O, Addr=$ipv6_ip')\n";
        }
        else {
            $smtpsPort = "dnl DAEMON_OPTIONS(`Port=smtps, Name=TLSMTA, M=s')\n";
        }

        # submission(587) port
        if ($obj->{enableSubmissionPort}) {
            $submissionPort = "DAEMON_OPTIONS(`Port=submission, Name=MSA, M=Ea')\n";
            $ipv6_part .= "DAEMON_OPTIONS(`Familiy=inet6, Port=submission, Name=MSA-v6, M=Ea, Modifier=O, Addr=$ipv6_ip')\n";
        }
        else {
            $submissionPort = "dnl DAEMON_OPTIONS(`Port=submission, Name=MSA, M=Ea')\n";
        }
    }
    elsif (($System->{gateway} eq "") && ($System->{gateway_IPv6} ne "")) {
        #
        ### IPv6 only:
        #
        # smtp port
        if ($obj->{enableSMTP}) {
            $smtpPort = "dnl DAEMON_OPTIONS(`Port=smtp, Name=MTA')\n";
            $ipv6_part .= "DAEMON_OPTIONS(`Familiy=inet6, port=smtp, Name=MTA-v6, Modifier=O')\n";
        }
        else {
            $smtpPort = "dnl DAEMON_OPTIONS(`Port=smtp, Name=MTA')\n";
        }

        # smtps port
        if ($obj->{enableSMTPS}) {
            $smtpsPort = "dnl DAEMON_OPTIONS(`Port=smtps, Name=TLSMTA, M=s')\n";
            $ipv6_part .= "DAEMON_OPTIONS(`Familiy=inet6, Port=smtps, Name=TLSMTA-v6, M=s, Modifier=O')\n";
        }
        else {
            $smtpsPort = "dnl DAEMON_OPTIONS(`Port=smtps, Name=TLSMTA, M=s')\n";
        }

        # submission(587) port
        if ($obj->{enableSubmissionPort}) {
            $submissionPort = "dnl DAEMON_OPTIONS(`Port=submission, Name=MSA, M=Ea')\n";
            $ipv6_part .= "DAEMON_OPTIONS(`Familiy=inet6, Port=submission, Name=MSA-v6, M=Ea, Modifier=O')\n";
        }
        else {
            $submissionPort = "dnl DAEMON_OPTIONS(`Port=submission, Name=MSA, M=Ea')\n";
        }
    }
    else {
        #
        ### IPv4 only:
        #
        # smtp port
        if ($obj->{enableSMTP}) {
            $smtpPort = "DAEMON_OPTIONS(`Port=smtp, Name=MTA')\n";
        }
        else {
            $smtpPort = "dnl DAEMON_OPTIONS(`Port=smtp, Name=MTA')\n";
        }

        # smtps port
        if ($obj->{enableSMTPS}) {
            $smtpsPort = "DAEMON_OPTIONS(`Port=smtps, Name=TLSMTA, M=s')\n";
        }
        else {
            $smtpsPort = "dnl DAEMON_OPTIONS(`Port=smtps, Name=TLSMTA, M=s')\n";
        }

        # submission(587) port
        if ($obj->{enableSubmissionPort}) {
            $submissionPort = "DAEMON_OPTIONS(`Port=submission, Name=MSA, M=Ea')\n";
        }
        else {
            $submissionPort = "dnl DAEMON_OPTIONS(`Port=submission, Name=MSA, M=Ea')\n";
        }
    }

    select $out;
    while (<$in>) {
        if (/^dnl DAEMON_OPTIONS\(\`Port=smtp, Name=MTA'/o || /^DAEMON_OPTIONS\(\`Port=smtp, Name=MTA'/o ) {
            print $smtpPort;
        } elsif (/^dnl DAEMON_OPTIONS\(\`Port=smtps, Name=TLSMTA,/o || /^DAEMON_OPTIONS\(\`Port=smtps, Name=TLSMTA,/o ) {
            print $smtpsPort;
        } elsif (/^dnl DAEMON_OPTIONS\(\`Port=submission, Name=MSA,/o || /^DAEMON_OPTIONS\(\`Port=submission, Name=MSA,/o ) {
            print $submissionPort;
        } elsif (/^DAEMON_OPTIONS\(\`Familiy=inet6/) {
            # We remove any existing IPv6 DAEMON_OPTIONS lines first (and add the correct ones later).
            next;
        } elsif (/^dnl DAEMON_OPTIONS\(\`port=smtp,Addr=::1, Name=MTA-v6, Family=inet6'\)dnl/) {
            # Insert IPv6 related lines below this marker:
            print "dnl DAEMON_OPTIONS(`port=smtp,Addr=::1, Name=MTA-v6, Family=inet6')dnl\n";
            # Insert IPv6 related lines (if there are any to insert):
            print $ipv6_part;
        } else {
            print $_;
        }
    }
    return 1;
}

sub debug_msg {
    if ($DEBUG) {
    my $msg = shift;
    $DEBUG && print STDERR "$ARGV[0]: ", $msg, "\n";
    setlogsock('unix');
    openlog($0,'','user');
    syslog('info', "$ARGV[0]: $msg");
    closelog;
    }
}

# 
# Copyright (c) 2008-2024 Michael Stauber, SOLARSPEED.NET
# Copyright (c) 2008-2024 Team BlueOnyx, BLUEONYX.IT
# Copyright (c) 2003 Sun Microsystems, Inc. 
# All Rights Reserved.
# 
# 1. Redistributions of source code must retain the above copyright 
#     notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#     notice, this list of conditions and the following disclaimer in 
#     the documentation and/or other materials provided with the 
#     distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its 
#     contributors may be used to endorse or promote products derived 
#     from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# 
# You acknowledge that this software is not designed or intended for 
# use in the design, construction, operation or maintenance of any 
# nuclear facility.
# 