#!/usr/bin/perl -I/usr/sausalito/perl -I/usr/sausalito/handlers/base/email
# $Id: user_disable.pl
#

# Debugging switch:
$DEBUG = "0";
if ($DEBUG) {
  use Sys::Syslog qw( :DEFAULT setlogsock);
}

use CCE;
use Email;
use Sauce::Service;
use Sauce::Util;

&debug_msg("Startup");

my $Access = $Email::ACCESS;
my $postfix_suspended_users = '/etc/postfix/suspended_users';

my $cce = new CCE;
$cce->connectfd();

my $oid = $cce->event_oid();

my ($ok, $user) = $cce->get($oid);
my ($ok, $email) = $cce->get($oid, "Email");
my ($ok, $user_disk) = $cce->get($oid, "Disk");

my $group = $user->{'site'};
my $username = $user->{'name'};

&debug_msg("Processing User $username. Over-Quota-Status: " . $user_disk->{over_quota} . "\n");

my @oid = $cce->find('Vsite', { 'name' => $group });
my ($ok, $domain) = $cce->get($oid[0]);
my ($ok, $domain_disk) = $cce->get($oid[0], "Disk");

my $vsite_overquota = 0;
if ($domain_disk->{'used'} >= $domain_disk->{'quota'}) {
  $vsite_overquota = 1;
}

my $virtualsite = $domain->{'fqdn'};

my @user_aliases =  $cce->scalar_to_array($email->{'aliases'});
push(@user_aliases, $username);

my @server_aliases = $cce->scalar_to_array($domain->{'mailAliases'});

push(@server_aliases, $virtualsite);
my $access_list;
my @emailList;
if (($user->{'emailDisabled'} eq "1") || (($user_disk->{over_quota} eq "1") || ($vsite_overquota eq "1"))) {
    foreach my $server(@server_aliases) {
        foreach my $user_name(@user_aliases) {
            if ($user_name) {
                push(@emailList, $user_name . '@' . $server);
            }
        }
    }
}

@emailList = nonDuplicatedArray(@emailList);
foreach my $entry(@emailList) {
    if (($user_disk->{over_quota} eq "1") || ($vsite_overquota eq "1")) {
        $access_list .= $entry . "\t\tERROR:5.2.2:550 User is over quota\n";
    }
    else {
        $access_list .= $entry . "\t\tERROR:5.1.1:550 User unknown\n";
    }
}

if (!Sauce::Util::replaceblock($Access,
    "### Start Block Email for User: $username on Virtual Site: $virtualsite ###", $access_list,
    "### END Block Email for User: $username on Virtual Site: $virtualsite ###")) {
    $cce->warn('[[base-email.cantEditFile]]', { 'file' => $Access });
    $cce->bye('FAIL');
    exit(1);
}
&debug_msg("Editing $Access done!");

if (-f $postfix_suspended_users) {

    &debug_msg("File $postfix_suspended_users exists.");
    $access_list = '';

    foreach my $entry(@emailList) {
        if (($user_disk->{over_quota} eq "1") || ($vsite_overquota eq "1")) {
            $access_list .= $entry . "\t\t550 5.2.2 User is over quota\n";
        }
        else {
            $access_list .= $entry . "\t\tREJECT\n";
        }
    }

    if (!Sauce::Util::replaceblock($postfix_suspended_users,
        "### Start Block Email for User: $username on Virtual Site: $virtualsite ###", $access_list,
        "### END Block Email for User: $username on Virtual Site: $virtualsite ###")) {
        $cce->warn('[[base-email.cantEditFile]]', { 'file' => $postfix_suspended_users });
        $cce->bye('FAIL');
        exit(1);
    }
    &debug_msg("Editing $postfix_suspended_users done!");
    system("postmap /etc/postfix/suspended_users");
}

$cce->bye("SUCCESS");
exit(0);

#
### Subs:
#

sub nonDuplicatedArray {
    my @Duplicated=@_;
    my %seen=();
    my (@NonDuplicatedArray,@Unique);
    @Unique = grep {! $seen{$_}++} @Duplicated;
    @NonDuplicatedArray = sort(@Unique);
    return @NonDuplicatedArray;
}

# Debug:
sub debug_msg {
    if ($DEBUG) {
        my $msg = shift;
        setlogsock('unix');
        openlog($0,'','user');
        syslog('info', "$ARGV[0]: $msg");
        closelog;
    }
}

# 
# Copyright (c) 2016-2024 Michael Stauber, SOLARSPEED.NET
# Copyright (c) 2016-2024 Team BlueOnyx, BLUEONYX.IT
# Copyright 2006, Brian Smith, NuOnce Networks, Inc.
# All Rights Reserved.
# 
# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its 
#    contributors may be used to endorse or promote products derived 
#    from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# 
# You acknowledge that this software is not designed or intended for 
# use in the design, construction, operation or maintenance of any 
# nuclear facility.
# 